#! /usr/bin/wish

proc mk_main_window {fp w InputFile} {
	###Window CTRL###
	wm title $w {LDesignLite}
	set fontFamily {Times New Roman}

	###Make Widget###
	frame [set fMain [mkPath $w fMain]]
	frame [set fMainIn [mkPath $fMain .fMainIn]]
	frame [set fMainCond [mkPath $fMain .fMainCond]]
	frame [set fMainResult [mkPath $fMain .fMainResult]]

	##Input##
	labelframe [set fInput [mkPath $w fInput]] -text {Input File} -font "{$fontFamily}"
	label [set lInput [mkPath $w lInput]] -text {Input File}
	labelframe [set lfInput [mkPath $w lbInput]] -text {}
	entry [set eInput [mkPath $w eInput]] -width 20 -state readonly -font "{$fontFamily}"
	button [set bInput [mkPath $w bInput]] -text {Reference} -font "{$fontFamily}"
	grid $eInput $bInput -in $lfInput
	message [set msInput [mkPath $w msInput]] -text {-} -width [$lfInput cget -width]
	$bInput configure -command "[list set_input_file_name $fp $eInput $msInput]"

#	grid $lInput $lfInput -in $fInput
#	grid ^ $msInput -in $fInput
	grid $lfInput -in $fInput -sticky w
	grid $msInput -in $fInput -sticky w

	##Condition##
	labelframe [set lfCond [mkPath $w lfCond]]

	frame [set fMode [mkPath $w fMode]]
	label [set lMode [mkPath $w lMode]] -text {Inheritance Mode} -font "{$fontFamily}"
	labelframe [set feMode [mkPath $w feMode]] -text {}
	entry [set eMode [mkPath $w eMode]] -width 10 -state readonly -font "{$fontFamily}"
	menubutton [set mbMode [mkPath $w mbMode]] -width 1 -direction left -text {}
	menu [set mMode [mkPath $mbMode .mMode]] -tearoff false
	$mbMode configure -menu $mMode
	grid $eMode $mbMode -in $feMode

	grid $lMode $feMode -in $fMode

	labelframe [set fPen [mkPath $w fPen]] -text {Penetrance} -font "{$fontFamily}"
	label [set lPenss [mkPath $w lPenss]] -text {RR} -font "{$fontFamily}"
	label [set lPensn [mkPath $w lPensn]] -text {RW} -font "{$fontFamily}"
	label [set lPennn [mkPath $w lPennn]] -text {WW} -font "{$fontFamily}"
	entry [set ePenss [mkPath $w ePenss]] -width 6 -font "{$fontFamily}"
	entry [set ePensn [mkPath $w ePensn]] -width 6 -font "{$fontFamily}"
	entry [set ePennn [mkPath $w ePennn]] -width 6 -font "{$fontFamily}"
	grid $lPenss $lPensn $lPennn -in $fPen
	grid $ePenss $ePensn $ePennn -in $fPen

	$mMode add command -label {Dominant} -command [list set_menu_element $eMode {Dominant} $ePenss $ePensn $ePennn]
	$mMode add command -label {Recessive} -command [list set_menu_element $eMode {Recessive} $ePenss $ePensn $ePennn]
	$mMode add command -label {Additive} -command [list set_menu_element $eMode {Additive} $ePenss $ePensn $ePennn]

	frame [set fRisk [mkPath $w fRisk]]
	label [set lRisk [mkPath $w lRisk]] -text {Risk Allele Frequency} -font "{$fontFamily}"
	entry [set eRisk [mkPath $w eRisk]] -width 8 -font "{$fontFamily}"
	grid $lRisk $eRisk -in $fRisk

	grid $fMode -in $lfCond -sticky w
	grid $fPen -in $lfCond
	grid $fRisk -in $lfCond -sticky w

	##Result##
	frame [set fResult [mkPath $w fResult]]
	label [set lResult [mkPath $w lResult]] -text {LOD score} -font "{$fontFamily}"
	entry [set eResult [mkPath $w eResult]] -width 8 -state readonly -font "{$fontFamily}"
	frame [set fAction [mkPath $w fAction]]
	button [set bGo [mkPath $w bGo]] -text {Go} -command "[list get_lod_score $fp $eInput $msInput $ePenss $ePensn $ePennn $eRisk $eResult]" -font "{$fontFamily}"
	button [set bClear [mkPath $w bClear]] -text {Clear} -command "[list clear_all $eInput $msInput $eResult]" -font "{$fontFamily}"
	grid $bGo $bClear -in $fAction

	grid $lResult $eResult $fAction -in $fResult


	##Packing##
	pack $fMain
	grid $fMainIn -in $fMain -sticky w
	grid $fMainCond -in $fMain -sticky w
	grid $fMainResult -in $fMain

	grid $fInput -in $fMainIn
	grid $lfCond -in $fMainCond
	grid $fResult -in $fMainResult

	
	##Initialize##
	$mMode invoke 0
	set_entry_value $eRisk {0.0005}

	global pFileName
	if {"{$pFileName}" ne {{}}} {
		set tpFileName [string map {\\ /} [lindex $pFileName 0]]
		set_ro_entry_value $eInput $tpFileName
		command_fn $fp {INFILE} [list $tpFileName $msInput]
	}

	return
}


proc set_input_file_name {fp wInput msInput} {
	set fileTypeList {{{Ped file} {.ped}} {{All} {*}}}

	set filePath [tk_getOpenFile -filetypes $fileTypeList] 
	if {$filePath ne ""} {
		set_ro_entry_value $wInput $filePath
		$wInput xview moveto [expr 1.0 - [lindex [$wInput xview] 1]]
		
		command_fn $fp {INFILE} [list $filePath $msInput]
	}


	return
}


proc set_menu_element {wMode value wss wsn wnn} {
	set_ro_entry_value $wMode $value

	set penSS [$wss get]
	if {$penSS eq {}} {
		set penSS {0.9999}
		set_entry_value $wss $penSS
	}
	set penNN [$wnn get]
	if {$penNN eq {}} {
		set penNN {0.0001}
		set_entry_value $wnn $penNN
	}

	if {$value eq {Dominant}} {
		set penSN $penSS
	} elseif {$value eq {Recessive}} {
		set penSN $penNN
	} elseif {$value eq {Additive}} {
		set penSN [expr [expr $penSS + $penNN] / 2.0]
	}

	set_entry_value $wsn $penSN

	return
}


proc get_lod_score {fp wInput wMessage wPenSS wPenSN wPenNN wRisk wResult} {
	set inputPath [$wInput get]
	set fileFlg [$wMessage cget -text]
	set penSS [$wPenSS get]
	set penSN [$wPenSN get]
	set penNN [$wPenNN get]
	set raf [$wRisk get]

	set args [list $inputPath $penSS $penSN $penNN $raf $wResult]

	set args [check_value $fileFlg $wMessage $args {wordchar} [list "{$fileFlg}" eq {{OK}}]]
	set args [check_value $penSS $wPenSS $args {double} [list  0 <= $penSS && $penSS <= 1.0]]
	set args [check_value $penSN $wPenSS $args {double} [list  0 <= $penSN && $penSN <= 1.0]]
	set args [check_value $penNN $wPenSS $args {double} [list  0 <= $penNN && $penNN <= 1.0]]
	set args [check_value $raf $wRisk $args {double} [list  0 <= $raf && $raf <= 1.0]]

	if {[llength $args]} {
		command_fn $fp {CALC_MINI} $args
	}

	return
}


proc clear_all {wInput wMessage wResult} {
	set_ro_entry_value $wInput {}
	set_ro_entry_value $wResult {}

	$wMessage configure -text {-}

	return
}


proc check_value {value widget args class cond} {
	global defGB

#	puts "FLG:[string is $class -strict $value]:COND:$cond"
	if {[string is $class -strict $value] == 1 && [expr $cond] == 1} {
		$widget configure -background "$defGB"
	} else {
		set args {}
		$widget configure -background {red}
	}

	return $args
}

