/**
 * Linkage Analysis
 * @author Ryo TAKEMURA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "fast_linkage_model.h"
#include "fast_linkage.h"


int main(int argc, char **argv){
	int flg[3];

	char *in_pre_file;
	char *in_genotype_file;
	char *in_penetrance_file;
	double risk_allele_freq;
	double *raf;//Risk Allele Frequency
	double *rgf;//Risk Genobype Frequency

	FAMILY_DATA *family_data;
	FAMILY_DATA *cur;
	double *penetrance;

	//double lod_score;
	RLOD lod_score;

	SDATA *ordered_score;
	SDATA *ocur;

	LINKAGE *linkage_result;
	LINKAGE *lcur;

	int i, j;
       

	in_pre_file = NULL;
	in_genotype_file = NULL;
	in_penetrance_file = NULL;
	risk_allele_freq = -1;
	for(i = 0; i < 3; i++){
		flg[i] = 0;
	}
	for(i = 1; i < argc; i++){
		switch (i){
			case 1:
				in_pre_file = argv[i];
				break;
			case 2:
				in_genotype_file = argv[i];
				break;
			case 3:
				in_penetrance_file = argv[i];
				break;
			case 4:
				sscanf(argv[i], "%lf", &risk_allele_freq);
				break;
			default:
				if(strcmp(argv[i], "LOD") == 0){
					flg[0] = 1;
				}
				else if(strcmp(argv[i], "ORDER") == 0){
					flg[1] = 1;
				}
				else if(strcmp(argv[i], "LINKAGE") == 0){
					flg[2] = 1;
				}
				else if(strcmp(argv[i], "ALL") == 0){
					for(j = 0; j < 2; j++){
						flg[j] = 1;
					}
				}
				else{
					printf("Illigal Switch(%s)\n", argv[i]);
				}
				break;
		}
	}

	if(in_pre_file == NULL || in_genotype_file == NULL || in_penetrance_file == NULL || risk_allele_freq < 0){
		printf("Illigal Parametaer Format...\n");
		exit(0);
	}
	if(flg[0] == 0 && flg[1] == 0){
		flg[0] = 1;
	}

	printf("PRE:%s\nGEN:%s\nPEN:%s\nRISK:%lf\n", in_pre_file, in_genotype_file, in_penetrance_file, risk_allele_freq);

	if((family_data = get_pedigree_data(in_pre_file)) == NULL){
		printf("Error in get_pedigree data...\n");
		exit(0);
	}

	if((family_data = get_genotype_data(in_genotype_file, family_data)) == NULL){
		printf("Error in get_genotype_data...\n");
		exit(0);
	}

	if((penetrance = get_penetrance_data(in_penetrance_file)) == NULL){
		printf("Error in get_penetrance_data...\n");
		exit(0);
	}

	if((raf = get_risk_allele_frequency(risk_allele_freq)) == NULL){
		printf("Error in get_risk_allele_frqquency...\n");
		exit(0);
	}

	if((rgf = get_risk_genotype_frequency(risk_allele_freq)) == NULL){
		printf("Error in get_risk_allele_frqquency...\n");
		exit(0);
	}

	printf("ID,Fid,Mid,SEX,PHENOTYPE,GEN,CHILD\n");
	cur = family_data->next;
	while(cur != NULL){
		printf("%s,%s,%s,%d,%d,%lf,%lf,%lf", cur->id, cur->Fid == NULL ? "0" : cur->Fid->id, cur->Mid == NULL ? "0" : cur->Mid->id, cur->SEX, cur->PHENOTYPE, cur->GENOTYPE[0], cur->GENOTYPE[1], cur->GENOTYPE[2]);
		for(i = 0; i < cur->num_of_childs; i++){
			printf(",%s", cur->childs[i]->id);
		}
		printf("\n");

		cur = cur->next;
	}
	printf("PENETRANCE:%lf/%lf/%lf\n", penetrance[0], penetrance[1], penetrance[2]);
	printf("RISK_ALLELE:%lg/%lg\n", raf[0], raf[1]);
	printf("RISK_GENOTYPE:%lg/%lg/%lg\n", rgf[0], rgf[1], rgf[2]);

	if(flg[0] == 1){
		lod_score = get_LOD_score(family_data, penetrance, raf, rgf, 0);

		printf("LOD score:%lg\n", lod_score.LOD_score);
		printf("(ALT:%lg / NULL:%lg)\n", lod_score.alt, lod_score.null);
	}
	if(flg[1] == 1){
		ordered_score = get_ordered_score(family_data, penetrance, raf, rgf);

		printf("[ORDER]\n");
		ocur = ordered_score->next;
		while(ocur != NULL){
			printf("%lf", ocur->LOD_score);
			printf(" (%s", ocur->element[0]);
			for(i = 1; i < ocur->num_of_element; i++){
				printf(",%s", ocur->element[i]);
			}
			printf(")");
			if(ocur->num_of_exclude > 0){
				printf("[");
				printf("%s", ocur->exclude_element[0]);
				for(i = 1; i < ocur->num_of_exclude; i++){
					printf(",%s", ocur->exclude_element[i]);
				}
				printf("] ");
			}
			printf("{%lg}\n", ocur->LOD_score / ordered_score->next->LOD_score);

			ocur = ocur->next;
		}
	}
	if(flg[2] == 1){
		linkage_result = get_linkage_analysis(family_data, penetrance, raf, rgf, 0.00001);

		printf("[RESULT]\n");
		lcur = linkage_result->next;
		while(lcur != NULL){
			printf("%lg,%lg\n", lcur->rf, lcur->LOD.LOD_score);

			lcur = lcur->next;
		}
	}

	return(0);
}


LINKAGE *get_linkage_analysis(FAMILY_DATA *fd, double *pen, double *raf, double *rgf, double min_digit){
	LINKAGE *result;
	LINKAGE *cur;
	LINKAGE *tmp;
	LINKAGE *init;
	LINKAGE **stack;
	int max_stack;
	int sp;

	double Ls[11];
	double dLs[10];

	double unit;

	int i;


	if((result = (LINKAGE *)malloc(sizeof(LINKAGE))) == NULL){
		return(NULL);
	}

	if((result->next = (LINKAGE *)malloc(sizeof(LINKAGE))) == NULL){
		return(NULL);
	}

	//result = result->next;
	result->next->next = NULL;

	result->next->rf = 0;
	result->next->LOD = get_LOD_score(fd, pen, raf, rgf, 0);
	Ls[0] = result->next->LOD.LOD_score;

	unit = 0.1;
	cur = result->next;
	for(i = 1; i <= 5 ; i++){
		if((cur->next = (LINKAGE *)malloc(sizeof(LINKAGE))) == NULL){
			return(NULL);
		}

		cur = cur->next;
		cur->next = NULL;

		cur->rf = i * unit;
		cur->LOD = get_LOD_score(fd, pen, raf, rgf, cur->rf);

		Ls[i] = cur->LOD.LOD_score;
	}

	cur = result->next;
	for(i = 0; i < 5; i++, cur = cur->next){
		dLs[i] = Ls[i + 1] - Ls[i];
	}

	max_stack = 10;
	if((stack = (LINKAGE **)malloc(sizeof(LINKAGE *) * max_stack)) == NULL){
		return(NULL);
	}

	sp = 0;
	cur = result->next;
	for(i = 0; i < 4; i++){
		if(dLs[i] >= 0 && dLs[i + 1] <= 0){
			stack[sp++] = cur;

			if(sp == max_stack){
				max_stack += max_stack;

				if((stack = (LINKAGE **)realloc(stack, sizeof(LINKAGE *) * max_stack)) == NULL){
					return(NULL);
				}
			}
		}

		cur = cur->next;
	}
	if(dLs[0] < 0){
		stack[sp++] = result->next;

		if(sp == max_stack){
			max_stack += max_stack;

			if((stack = (LINKAGE **)realloc(stack, sizeof(LINKAGE *) * max_stack)) == NULL){
				return(NULL);
			}
		}
	}
	if(dLs[4] > 0){
		stack[sp++] = cur;

		if(sp == max_stack){
			max_stack += max_stack;

			if((stack = (LINKAGE **)realloc(stack, sizeof(LINKAGE *) * max_stack)) == NULL){
				return(NULL);
			}
		}
	}

	while(sp > 0){
		init = stack[--sp];

		unit = (init->next->rf - init->rf) * 0.1;
		if(unit < min_digit){
			continue;
		}

		printf("RF:%lg(%lg)\n", init->rf, unit);

		cur = init;
		Ls[0] = cur->LOD.LOD_score;
		for(i = 1; i < 10; i++){
			if((tmp = (LINKAGE *)malloc(sizeof(LINKAGE))) == NULL){
				return(NULL);
			}
			tmp->next = cur->next;
			cur->next = tmp;

			tmp = cur;
			cur = cur->next;

			cur->rf = init->rf + unit * i;
			cur->LOD = get_LOD_score(fd, pen, raf, rgf, cur->rf);

			Ls[i] = cur->LOD.LOD_score;
		}

		for(i = 0; i < 9; i++){
			dLs[i] = Ls[i + 1] - Ls[i];
		}
		cur = result->next;
		for(i = 0; i < 9; i++){
			if(dLs[i] >= 0 && dLs[i + 1] <= 0){
				stack[sp++] = cur;

				if(sp == max_stack){
					max_stack += max_stack;

					if((stack = (LINKAGE **)realloc(stack, sizeof(LINKAGE *) * max_stack)) == NULL){
						return(NULL);
					}
				}
			}

			cur = cur->next;
		}
		if(dLs[0] < 0){
			stack[sp++] = init;

			if(sp == max_stack){
				max_stack += max_stack;

				if((stack = (LINKAGE **)realloc(stack, sizeof(LINKAGE *) * max_stack)) == NULL){
					return(NULL);
				}
			}
		}
		if(dLs[8] > 0){
			stack[sp++] = cur;

			if(sp == max_stack){
				max_stack += max_stack;

				if((stack = (LINKAGE **)realloc(stack, sizeof(LINKAGE *) * max_stack)) == NULL){
					return(NULL);
				}
			}
		}
	}

	return(result);
}


SDATA *get_ordered_score(FAMILY_DATA *fd, double *pen, double *raf, double *rgf){
	SDATA *result;
	SDATA *rcur;
	SDATA *trcur;
	//int rnum;
	SDATA *rtarget;
	double even_lod;

	FAMILY_DATA *cur;
	FAMILY_DATA *new;
	FAMILY_DATA *ncur;
	FAMILY_DATA *tcur;

	FAMILY_DATA *fd_pool;
	FAMILY_DATA *pool_last;
	int num_of_pool;
	int num_of_rent;

	RLOD score;

	int num_of_exclude;
	int max_exclude;
	int texclude;

	int v;
	unsigned long int vt;
	unsigned long int max_v;
	unsigned long int flg;

	int i, j;

	if((result = (SDATA *)malloc(sizeof(SDATA))) == NULL){
		return(NULL);
	}
	result->next = NULL;

	cur = fd->next;
	max_v = 1;
	while(cur != NULL){
		max_v *= 2;
		cur = cur->next;
	}
	/*
	printf("MAX_V*%ld\n", max_v);
	*/
	
	if((fd_pool = (FAMILY_DATA *)malloc(sizeof(FAMILY_DATA))) == NULL){
		return(NULL);
	}
	fd_pool->next = NULL;
	num_of_pool = 0;
	num_of_rent = 0;

	if((rcur = (SDATA *)malloc(sizeof(SDATA))) == NULL){
		return(NULL);
	}
	rcur->next = NULL;
	//rnum = 0;
	rcur->num_of_element = 0;
	//max_exclude = -1;
	max_exclude = 0;
	rcur->num_of_exclude = 0;
	rcur->exclude = 0;
	cur = fd->next;
	while(cur != NULL){
		rcur->num_of_element++;
		max_exclude++;
		cur = cur->next;
	}
	max_exclude = (max_exclude + (max_exclude % 2)) / 2;
	if((rcur->element = (char **)malloc(sizeof(char *) * rcur->num_of_element)) == NULL){
		return(NULL);
	}
	i = 0;
	cur = fd->next;
	while(cur != NULL){
		rcur->element[i] = cur->id;
		i++;

		cur = cur->next;
	}
	v = 0;
	/*
	printf("ORDER:%d", v);
	for(i = 0; i < rcur->num_of_element; i++){
		printf(",%s", rcur->element[i]);
	}
	printf("\n");
	*/

	score = get_LOD_score(fd, pen, raf, rgf, 0);
	/*
	printf("LOD:%lg:%lg:%lg\n", score.alt, score.null, score.LOD_score);
	*/
	rcur->LOD_score = score.LOD_score;
	trcur = result;
	while(trcur->next != NULL){
		if(rcur->LOD_score >= trcur->next->LOD_score){
			break;
		}

		trcur = trcur->next;
	}
	rcur->next = trcur->next;
	trcur->next = rcur;

	
	if((new = (FAMILY_DATA *)malloc(sizeof(FAMILY_DATA))) == NULL){
		return(NULL);
	}

	for(num_of_exclude = 1; num_of_exclude <= max_exclude; num_of_exclude++){
		printf("EXCLUDE:%d/%d\n", num_of_exclude, max_exclude);
		rtarget = result->next;
		while(rtarget != NULL){
			//printf("EACHEXCLUDE:%ld\n", rtarget->exclude);
			if(rtarget->num_of_exclude != num_of_exclude - 1){
				rtarget = rtarget->next;
				continue;
			}

			even_lod = rtarget->LOD_score;
			break;
		}

		rtarget = result->next;
		while(rtarget != NULL){
			if(rtarget->num_of_exclude != num_of_exclude - 1 || rtarget->LOD_score != even_lod){
				rtarget = rtarget->next;
				continue;
			}

			for(v = 0; (1 << v) < max_v; v++){
				//vt = v;

				if((((rtarget->exclude >> v)) & 1) == 1){
					continue;
				}

				texclude = rtarget->exclude + (1 << v);
				trcur = result->next;
				while(trcur != NULL){
					if(trcur->exclude == texclude){
						break;
					}

					trcur = trcur->next;
				}

				if(trcur != NULL){
					continue;
				}

				ncur = new;
				if((rcur = (SDATA *)malloc(sizeof(SDATA))) == NULL){
					return(NULL);
				}
				rcur->next = NULL;
				//rnum = 0;
				rcur->num_of_element = 0;
				rcur->num_of_exclude = num_of_exclude;
				rcur->exclude = rtarget->exclude + (1 << v);

				vt = max_v - (rcur->exclude + 1);
				//printf("EXCLUDE:vt(%ld) = max_v(%ld) - (EXCLUDE(%ld) + 1)\n", vt, max_v, rcur->exclude);

				cur = fd->next;
				while(cur != NULL){
					flg = vt % 2;
					vt = (vt - flg) / 2;

					if(num_of_pool == 0){
						if((ncur->next = (FAMILY_DATA *)malloc(sizeof(FAMILY_DATA))) == NULL){
							return(NULL);
						}
						ncur = ncur->next;
						ncur->id = NULL;
						ncur->max_childs = 5;
						if((ncur->childs = (FAMILY_DATA **)malloc(sizeof(FAMILY_DATA *) * ncur->max_childs)) == NULL){
							return(NULL);
						}

						num_of_rent++;
					}
					else{
						ncur->next = fd_pool->next;
						fd_pool->next = fd_pool->next->next;
						num_of_pool--;
						num_of_rent++;

						ncur = ncur->next;
					}

					ncur->next = NULL;

					/*
					if((ncur->id = (char *)realloc(ncur->id, sizeof(cur->id))) == NULL){
						return(NULL);
					}
					strcpy(ncur->id, cur->id);
					*/
					ncur->id = cur->id;

					ncur->SEX = cur->SEX;
					ncur->PHENOTYPE = cur->PHENOTYPE;
					/*
					printf("FLG:%ld/%ld/%ld\n", vt, flg, max_v);
					*/
					if(flg == 1){
						for(i = 0; i < 3; i++){
							ncur->GENOTYPE[i] = cur->GENOTYPE[i];
						}

						//rnum++;
						rcur->num_of_element++;
					}
					else{
						for(i = 0; i < 3; i++){
							ncur->GENOTYPE[i] = 0;
						}

						//rcur->num_of_exclude++;
					}
					ncur->num_of_childs = 0;

					cur = cur->next;
				}

				ncur = new->next;
				while(ncur != NULL){
					cur = fd->next;
					while(cur != NULL){
						if(strcmp(ncur->id, cur->id) == 0){
							break;
						}

						cur = cur->next;
					}

					tcur = new->next;
					while(tcur != NULL){
						if(cur->Fid != NULL && strcmp(tcur->id, cur->Fid->id) == 0){
							break;
						}

						tcur = tcur->next;
					}

					ncur->Fid = tcur;

					tcur = new->next;
					while(tcur != NULL){
						if(cur->Mid != NULL && strcmp(tcur->id, cur->Mid->id) == 0){
							break;
						}

						tcur = tcur->next;
					}

					ncur->Mid = tcur;

					ncur = ncur->next;
				}

				cur = new->next;
				while(cur != NULL){
					if(cur->Fid != NULL){
						if(cur->Fid->num_of_childs >= cur->Fid->max_childs){
							cur->Fid->max_childs += cur->Fid->max_childs;
							if((cur->Fid->childs = (FAMILY_DATA **)realloc(cur->Fid->childs, sizeof(FAMILY_DATA *) * cur->Fid->max_childs)) == NULL){
								printf("Flag8\n");
								return(NULL);
							}
						}

						cur->Fid->childs[cur->Fid->num_of_childs++] = cur;
					}
					if(cur->Mid != NULL){
						if(cur->Mid->num_of_childs >= cur->Mid->max_childs){
							cur->Mid->max_childs += cur->Mid->max_childs;
							if((cur->Mid->childs = (FAMILY_DATA **)realloc(cur->Mid->childs, sizeof(FAMILY_DATA *) * cur->Mid->max_childs)) == NULL){
								printf("Flag9\n");
								return(NULL);
							}
						}

						cur->Mid->childs[cur->Mid->num_of_childs++] = cur;
					}

					cur = cur->next;
				}

				//rcur->num_of_element = rnum;
				if((rcur->element = (char **)malloc(sizeof(char *) * rcur->num_of_element)) == NULL){
					return(NULL);
				}
				if((rcur->exclude_element = (char **)malloc(sizeof(char *) * rcur->num_of_exclude)) == NULL){
					return(NULL);
				}
				i = 0;
				j = 0;
				cur = new->next;
				//vt = v;
				vt = max_v - (rcur->exclude + 1);
				while(cur != NULL){
					flg = vt % 2;
					vt = (vt - flg) / 2;

					if(flg == 1){
						/*
						if((rcur->element[i] = (char *)malloc(sizeof(cur->id))) == NULL){
							return(NULL);
						}
						strcpy(rcur->element[i], cur->id);
						*/
						rcur->element[i++] = cur->id;
					}
					else{
						rcur->exclude_element[j++] = cur->id;
					}

					cur = cur->next;
				}
				/*
				printf("ORDER:%d", v);
				for(i = 0; i < rcur->num_of_element; i++){
					printf(",%s", rcur->element[i]);
				}
				printf("\n");
				*/

				/*DEBUG
				ncur = new->next;
				while(ncur != NULL){
					printf("NEW CHECK:%s\n", ncur->id);
					if(ncur->Fid != NULL){
						printf("NEW CHECK Fid:%s\n", ncur->Fid->id);
					}
					else{
						printf("NEW CHECK Fid:NAN\n");
					}
					if(ncur->Mid != NULL){
						printf("NEW CHECK Mid:%s\n", ncur->Mid->id);
					}
					else{
						printf("NEW CHECK Mid:NAN\n");
					}
					printf("NEW CHECK GEN:%lf:%lf:%lf\n", ncur->GENOTYPE[0], ncur->GENOTYPE[1], ncur->GENOTYPE[2]);
					printf("NEW CHECK PEN:%d\n", ncur->PHENOTYPE);
					printf("NEW CHILD:%d\n", ncur->num_of_childs);
					ncur = ncur->next;
				}
				*/
				//rcur->LOD_score = get_LOD_score(new, pen);
		
				/*
				ncur = new->next;
				while(ncur != NULL){
					printf("CHECK:%s:%d:%lg/%lg/%lg(%lg)\n", ncur->id, ncur->PHENOTYPE, ncur->GENOTYPE[0], ncur->GENOTYPE[1], ncur->GENOTYPE[2], (ncur->GENOTYPE[0] + ncur->GENOTYPE[1] + ncur->GENOTYPE[2]));
					ncur = ncur->next;
				}
				*/
				
/*				
	printf("GET_LOD_ID,Fid,Mid,SEX,PHENOTYPE,GEN,CHILD\n");
	cur = new->next;
	while(cur != NULL){
		printf("%s,%s,%s,%d,%d,%lf,%lf,%lf", cur->id, cur->Fid == NULL ? "0" : cur->Fid->id, cur->Mid == NULL ? "0" : cur->Mid->id, cur->SEX, cur->PHENOTYPE, cur->GENOTYPE[0], cur->GENOTYPE[1], cur->GENOTYPE[2]);
		for(i = 0; i < cur->num_of_childs; i++){
			printf(",%s", cur->childs[i]->id);
		}
		printf("\n");

		cur = cur->next;
	}
	printf("PENETRANCE:%lf/%lf/%lf\n", pen[0], pen[1], pen[2]);
	printf("RISK_ALLELE:%lg/%lg\n", raf[0], raf[1]);
	printf("RISK_GENOTYPE:%lg/%lg/%lg\n", rgf[0], rgf[1], rgf[2]);
*/

				score = get_LOD_score(new, pen, raf, rgf, 0);
				//printf("LOD:%lg:%lg:%lg\n", score.alt, score.null, score.LOD_score);
				rcur->LOD_score = score.LOD_score;
				trcur = result;
				while(trcur->next != NULL){
					if(rcur->LOD_score == trcur->next->LOD_score){
						if(rcur->num_of_element > trcur->next->num_of_element){
							break;
						}
					}
					else if(rcur->LOD_score > trcur->next->LOD_score){
						break;
					}

					trcur = trcur->next;
				}
				rcur->next = trcur->next;
				trcur->next = rcur;

				pool_last = fd_pool;
				while(pool_last->next != NULL){
					pool_last = pool_last->next;
				}
				pool_last->next = new->next;
				num_of_pool += num_of_rent;
				num_of_rent = 0;
			}

			rtarget = rtarget->next;
		}
	}

	/*
	rcur = result->next;
	while(rcur != NULL){
		printf("RES:%lg", rcur->LOD_score);
		for(i = 0; i < rcur->num_of_element; i++){
			printf(",%s", rcur->element[i]);
		}
		printf("\n");

		rcur = rcur->next;
	}
	*/

	return(result);
}


RLOD get_LOD_score(FAMILY_DATA *fd, double *pen, double *raf, double *rgf, double alt_theta){
	DIP *dip;
	int number;
	ADATA *founder;
	ADATA *child;

	FAMILY_DATA *cur;

	/*
	double result;
	double alt_likelihood;
	double null_likelihood;
	*/
	RLOD result;

	int cnt;
	int ccnt;

	int i, j;


	//num_of_ind = cnt_pedigree(fd->next);
	if((dip = (DIP *)malloc((sizeof(DIP) * cnt_pedigree(fd->next)))) == NULL){
		printf("Error in Memory Allocate DIP...\n");
		exit(0);
	}

	cur = fd->next;
	while(cur != NULL){
		if(cur->GENOTYPE[0] == 0 && cur->GENOTYPE[1] == 0 && cur->GENOTYPE[2] == 0){
			if(cur->Fid == NULL || cur->Mid == NULL){
				for(i = 0; i < 3; i++){
					cur->GENOTYPE[i] = rgf[i];
				}
				cur->GENOTYPE[1] += cur->GENOTYPE[1];
			}
			else{
				for(i = 0; i < 3; i++){
					cur->GENOTYPE[i] = 1.0 / 3.0;
				}
			}
		}

		for(i = 0; i < 3; i++){
			for(j = 0; j < 3; j++){
				cur->calculate[i][j] = cur->GENOTYPE[i] * (cur->PHENOTYPE == 2 ? pen[j]: (1 - pen[j]));
			}
		}

		cur = cur->next;
	}

	number = 0;
	for(ccnt = 1; ccnt <= 3; ccnt++){
		cur = fd->next;
		while(cur != NULL){
			cnt = 0;
			for(i = 0; i < 3; i++){
				if(cur->GENOTYPE[i] != 0){
					cnt++;
				}
			}
			
			if(cnt == ccnt){
				//printf("DIP:%s:%d <> %d\n", cur->id, cnt, ccnt);
				cur->dip = number;
				dip[number++].data = cur;
			}

			cur = cur->next;
		}
	}

	founder = get_founder(dip, number);
	child = get_child(dip, number, founder);

	/*
	printf("FOUNDER");
	for(i = 0; i < founder->number; i++){
		printf(",%d", founder->data[i]);
	}
	printf("\nCHILD");
	for(i = 0; i < child->number; i++){
		printf(",%d", child->data[i]);
	}
	printf("\n");
	*/

	/*
	alt_likelihood = get_likelihood(fd, dip, founder, child, number, pen, 0);
	null_likelihood = get_likelihood(fd, dip, founder, child, number, pen, 0.5);
	result = log(alt_likelihood / null_likelihood) / log(10.0);
	*/
	result.alt = get_likelihood(fd, dip, founder, child, number, pen, alt_theta, raf, rgf);
	result.null = get_likelihood(fd, dip, founder, child, number, pen, 0.5, raf, rgf);
	result.LOD_score = log10(result.alt / result.null);//log(result.alt / result.null) / log(10.0);

	//printf("ALT:%lg\nNULL:%lg\nLOD score:%lg\n", alt_likelihood, null_likelihood, result);

	free(dip);
	free(founder->data);
	free(founder);
	free(child->data);
	free(child);

	return(result);
}


double get_likelihood(FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	RDATA res;
	double result;

	int i;


	res.max = 100;
	if((res.result = (double *)calloc(res.max, sizeof(double))) == NULL){
		return(0);
	}

	//get_likelihood_fnd_rec(0, &res, 1.0, fd, dip, founder, child, number, pen, theta, raf, rgf);
	//get_likelihood_fnd_loop(0, &res, 1.0, fd, dip, founder, child, number, pen, theta, raf, rgf);
	//get_likelihood_unite(&res, fd, dip, founder, child, number, pen, theta, raf, rgf);
	//get_likelihood_fool(0, 0, &res, fd, dip, founder, child, number, pen, theta, raf, rgf);
	get_likelihood_fnd_fool_rec(0, &res, 1.0, fd, dip, founder, child, number, pen, theta, raf, rgf);

	result = 0;
	for(i = res.max - 1; i >= 0; i--){
		result += res.result[i];
	}
	
	free(res.result);

	return(result);
}


void get_likelihood_child_fool_rec(int digit, RDATA *res, double elv, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	int id;
	int Fid;
	int Mid;

	int fv[2];
	int mv[2];
	int flimit[2] = {2, 2};
	int mlimit[2] = {2, 2};
	double frcomb[2] = {theta, (1 - theta)};
	double mrcomb[2] = {theta, (1 - theta)};
	int dammy_g[2] = {0, 1};
	int dammy_p[2] = {0, 1};
	double rc;

	int dg;
	int tdg;

	double telv;
	//double ttelv[2];

	double multi_rcomb[2][2];
	int i, j;

	int diplo_index[2];
	int diplo[16];
	int k, l;


	if(digit == child->number){
		dg = (int)log10(elv);
		dg = dg < 0 ? abs(dg) + 1 : 0;
		if(dg >= res->max){
			if((res->result = (double *)realloc(res->result, sizeof(double) * (dg + 1))) == NULL){
				return;
			}

			for(; res->max < dg + 1; res->max++){
				res->result[res->max] = 0;
			}
		}

		res->result[dg] += elv;
		tdg = (int)log10(elv);
		tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		while(dg != tdg){
			res->result[tdg] += res->result[dg];
			res->result[dg] = 0;

			dg = tdg;
			tdg = (int)log(dg);
			tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		}
		//printf("ADD RESULT(CHD):%d <> %d:%lg:%lg:%lg:%lg:%lg:%lg:%d\n", digit, child->number - 1, telv, res->result[dg], dip[id].data->GENOTYPE[dip[id].G], dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]), frcomb[fv[0] == fv[1] ? 1 : 0], mrcomb[mv[0] == mv[1] ? 1 : 0], dg);

		return;
	}

	for(i = 0; i < 16; i++){
		diplo[i] = 0;
	}

	id = child->data[digit];
	Fid = dip[id].data->Fid == NULL ? -1 : dip[id].data->Fid->dip;
	Mid = dip[id].data->Mid == NULL ? -1 : dip[id].data->Mid->dip;

	if(Fid != -1){
		if(dip[Fid].D[0][0] == dip[Fid].D[1][0]){
			flimit[0] = 1;
			rc = frcomb[0] + frcomb[1];
			frcomb[0] = rc;
			frcomb[1] = rc;
		}
		if(dip[Fid].D[0][1] == dip[Fid].D[1][1]){
			flimit[1] = 1;
			rc = frcomb[0] + frcomb[1];
			frcomb[0] = rc;
			frcomb[1] = rc;
		}
	}
	if(Mid != -1){
		if(dip[Mid].D[0][0] == dip[Mid].D[1][0]){
			mlimit[0] = 1;
			rc = mrcomb[0] + mrcomb[1];
			mrcomb[0] = rc;
			mrcomb[1] = rc;
		}
		if(dip[Mid].D[0][1] == dip[Mid].D[1][1]){
			mlimit[1] = 1;
			rc = mrcomb[0] + mrcomb[1];
			mrcomb[0] = rc;
			mrcomb[1] = rc;
		}
	}

	for(i = 0; i < 2; i++){
		for(j = 0; j < 2; j++){
			multi_rcomb[i][j] = frcomb[i] * mrcomb[j];
		}
	}

	for(fv[0] = 0; fv[0] < flimit[0]; fv[0]++){
		dip[id].D[0][0] = Fid != -1 ? dip[Fid].D[fv[0]][0] : dammy_g[fv[0]];
		for(mv[0] = 0; mv[0] < mlimit[0]; mv[0]++){
			dip[id].D[1][0] = Mid != -1 ? dip[Mid].D[mv[0]][0] : dammy_g[mv[0]];

			dip[id].G = dip[id].D[0][0] + dip[id].D[1][0];
			if(dip[id].data->GENOTYPE[dip[id].G] == 0){
				continue;
			}

			for(fv[1] = 0; fv[1] < flimit[1]; fv[1]++){
				dip[id].D[0][1] = Fid != -1 ? dip[Fid].D[fv[1]][1] : dammy_p[fv[1]];

				if((Fid != -1 ? frcomb[fv[0] == fv[1] ? 1 : 0] : raf[dip[id].D[0][1]]) == 0){
					continue;
				}

				for(mv[1] = 0; mv[1] < mlimit[1]; mv[1]++){
					dip[id].D[1][1] = Mid != -1 ? dip[Mid].D[mv[1]][1] : dammy_p[mv[1]];

					dip[id].P = dip[id].D[0][1] + dip[id].D[1][1];

					if((Mid != -1 ? mrcomb[mv[0] == mv[1] ? 1 : 0] : raf[dip[id].D[1][1]]) == 0 || dip[id].data->calculate[dip[id].G][dip[id].P] == 0){
						continue;
					}

					if(pen[dip[id].P] == 0){
						continue;
					}

					diplo_index[0] = 0;
					diplo_index[1] = 0;
					for(k = 0; k < 2; k++){
						for(l = 0; l < 2; l++){
							diplo_index[k] = diplo_index[k] * 2 + dip[id].D[k][l];
						}
					}
					diplo[diplo_index[diplo_index[0] < diplo_index[1] ? 0 : 1] * 4 + diplo_index[diplo_index[0] > diplo_index[1] ? 0 : 1]]++;
				}
			}
		}
	}

	for(i = 0; i < 16; i++){

		if(diplo[i] == 0){
			continue;
		}
	
		diplo_index[0] = i;
		for(k = 1; k >= 0; k--){
			for(l = 1; l >= 0; l--){
				dip[id].D[k][l] = diplo_index[0] % 2;
				diplo_index[0] = (diplo_index[0] - dip[id].D[k][l]) / 2;
			}
		}
		dip[id].G = dip[id].D[0][0] + dip[id].D[1][0];
		dip[id].P = dip[id].D[0][1] + dip[id].D[1][1];


		telv = elv * dip[id].data->calculate[dip[id].G][dip[id].P] * (Fid == -1 ? Mid == -1 ?  raf[dip[id].D[0][1]] * raf[dip[id].D[1][1]] : raf[dip[id].D[0][1]] : Mid == -1 ? raf[dip[id].D[1][1]] : multi_rcomb[fv[0] == fv[1] ? 1 : 0][mv[0] == mv[1] ? 1 : 0]) * diplo[i];

		get_likelihood_child_fool_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
	}

	return;
}


void get_likelihood_fnd_fool_rec(int digit, RDATA *res, double elv, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	int id;
	//double ttelv;
	double telv;
	int i, j;

	int diplo_index[2];
	int diplo[16];
	int k, l;


	if(digit == founder->number){
		get_likelihood_child_fool_rec(0, res, elv, fd, dip, founder, child, number, pen, theta, raf, rgf);

		return;
	}

	for(i = 0; i < 16; i++){
		diplo[i] = 0;
	}

	id = founder->data[digit];

	//ORDERED 0:AA 1:AB 2:BA 3:BB / GENOTYPE 0:AA 1:AB 2:BB / ALLELE 0:A 1:B
	for(i = 0; i < 4; i++){
		dip[id].G = i == 0 ? 0 : i == 3 ? 2 : 1;
		dip[id].D[0][0] = i == 0 ? 0 : i == 1 ? 0 : 1;
		dip[id].D[1][0] = i == 0 ? 0 : i == 2 ? 0 : 1;

		if(dip[id].data->GENOTYPE[dip[id].G] == 0){
			continue;
		}

		//ORDERED 0:NN 1:NS 2:SN 3:SS / GENOTYPE 0:NN 1:NS 2:SS /ALLELE 0:N 1:S
		for(j = 0; j < 4; j++){
			dip[id].P = j == 0 ? 0 : j == 3 ? 2 : 1;
			dip[id].D[0][1] = j == 0 ? 0 : j == 1 ? 0 : 1;
			dip[id].D[1][1] = j == 0 ? 0 : j == 2 ? 0 : 1;

			if(pen[dip[id].P] == 0){
				continue;
			}

			diplo_index[0] = 0;
			diplo_index[1] = 0;
			for(k = 0; k < 2; k++){
				for(l = 0; l < 2; l++){
					diplo_index[k] = diplo_index[k] * 2 + dip[id].D[k][l];
				}
			}
			diplo[diplo_index[diplo_index[0] < diplo_index[1] ? 0 : 1] * 4 + diplo_index[diplo_index[0] > diplo_index[1] ? 0 : 1]]++;
		}
	}

	for(i = 0; i < 16; i++){
		if(diplo[i] == 0){
			continue;
		}

		diplo_index[0] = i;
		for(k = 1; k >= 0; k--){
			for(l = 1; l >= 0; l--){
				dip[id].D[k][l] = diplo_index[0] % 2;
				diplo_index[0] = (diplo_index[0] - dip[id].D[k][l]) / 2;
			}
		}
		dip[id].G = dip[id].D[0][0] + dip[id].D[1][0];
		dip[id].P = dip[id].D[0][1] + dip[id].D[1][1];

		telv = diplo[i] * elv * rgf[dip[id].P] * dip[id].data->calculate[dip[id].G][dip[id].P];
		//printf("GEN FREQ:%d:%lg:%lg\n", dip[id].G, raf,(dip[id].G == 2 ? raf * raf : dip[id].G == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf)));

		if(telv == 0){
			continue;
		}

		get_likelihood_fnd_fool_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
	}

	return;
}

void get_likelihood_fool(int fdigit, int cdigit, RDATA *res, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	FAMILY_DATA *id;
	FAMILY_DATA *Fid;
	FAMILY_DATA *Mid;
	FAMILY_DATA *Cid;

	double pll;//Part of likelihood
	//double plt[7];//Tmp for Part of likelihood
	
	double fgp;
	double mgp;

	int H[2];
	int Hlimit[2] = {4, 4};

	int fH[2];
	int mH[2];
	int fHlimit[2] = {4, 4};
	int mHlimit[2] = {4, 4};
	int fv[2];
	int mv[2];
	int fvlimit[2] = {2, 2};
	int mvlimit[2] = {2, 2};

	int G[2];
	int FG[2];
	int MG[2];
	int P[2];
	int FP[2];
	int MP[2];

	int i;

	//printf("DIGIT:%d/%d-%d/%d\n", fdigit, cdigit, founder->number, child->number);
	if(fdigit < founder->number){
		get_likelihood_fool(fdigit + 1, cdigit, res, fd, dip, founder, child, number, pen, theta, raf, rgf);
		printf("FBACK:%d/%d-%d/%d\n", fdigit, cdigit, founder->number, child->number);
	}
	else{
		if(cdigit < child->number){
			get_likelihood_fool(fdigit, cdigit + 1, res, fd, dip, founder, child, number, pen, theta, raf, rgf);
			printf("CBACK:%d/%d-%d/%d\n", fdigit, cdigit, founder->number, child->number);
		}
		else{
			return;
		}
		id = dip[child->data[cdigit]].data;
		Fid = id->Fid;
		Mid = id->Mid;

		id->part_likelihood = 1.0;
//		for(i = 0; i < id->num_of_childs; i++){
//			Cid = id->childs[i];
//			id->part_likelihood *= Cid->part_likelihood;
//		}

		pll = 0;
		for(fH[0] = 0; fH[0] < fHlimit[0]; fH[0]++){
			FG[0] = fH[0] <= 1 ? 0 : 1;
			FG[1] = fH[0] % 2 == 0 ? 0 : 1;

			fgp = Fid != NULL ? Fid->GENOTYPE[FG[0] + FG[1]] : rgf[FG[0] + FG[1]];
			for(fH[1] = 0; fH[1] < fHlimit[1]; fH[1]++){
				FP[0] = fH[1] <= 1 ? 0 : 1;
				FP[1] = fH[1] % 2 == 0 ? 0 : 1;
				for(mH[0] = 0; mH[0] < mHlimit[0]; mH[0]++){
					MG[0] = mH[0] <= 1 ? 0 : 1;
					MG[1] = mH[0] % 2 == 0 ? 0 : 1;

					mgp = Mid != NULL ? Mid->GENOTYPE[MG[0] + MG[1]] : rgf[MG[0] + MG[1]];
					for(mH[1] = 0; mH[1] < mHlimit[1]; mH[1]++){
						MP[0] = mH[1] <= 1 ? 0 : 1;
						MP[1] = mH[1] % 2 == 0 ? 0 : 1;
						for(fv[0] = 0; fv[0] < fvlimit[0]; fv[0]++){
							G[0] = FG[fv[0]];
							for(fv[1] = 0; fv[1] < fvlimit[1]; fv[1]++){
								P[0] = FP[fv[1]];
								for(mv[0] = 0; mv[0] < mvlimit[0]; mv[0]++){
									G[1] = MG[mv[0]];
									for(mv[1] = 0; mv[1] < mvlimit[1]; mv[1]++){
										P[1] = MP[mv[1]];

										//pll += fgp * mgp * id->GENOTYPE[G[0] + G[1]] * (id->PHENOTYPE == 2 ? pen[P[0] + P[1]] : (1.0 - pen[P[0] + P[1]])) * (fv[0] == fv[1] ? (1.0 - theta) : theta) * (mv[0] == mv[1] ? (1.0 - theta) : theta);
										pll += fgp * mgp * id->GENOTYPE[G[0] + G[1]] * rgf[P[0] + P[1]] * (id->PHENOTYPE == 2 ? pen[P[0] + P[1]] : (1.0 - pen[P[0] + P[1]])) * (fv[0] == fv[1] ? (1.0 - theta) : theta) * (mv[0] == mv[1] ? (1.0 - theta) : theta);
//										if(fgp > 0 && mgp > 0 && id->GENOTYPE[G[0] + G[1]] > 0){
//											printf("fH:%d(%d-%d)/%d:mH:%d(%d-%d)/%d:fv:%d/%d:mv:%d/%d:FGP:%lf:MGP:%lf:GEN:%lf:PEN:%lf:FVTHETA:%lf:MVTHETA:%lf:pll:%lf(%lf)\n", fH[0], FG[0], FG[1], fH[1], mH[0], MG[0], MG[1], mH[1], fv[0], fv[1], mv[0], mv[1], fgp, mgp, id->GENOTYPE[G[0] + G[1]], (id->PHENOTYPE == 2 ? pen[P[0] + P[1]] : (1.0 - pen[P[0] + P[1]])), (fv[0] == fv[1] ? (1.0 - theta) : theta), (mv[0] == mv[1] ? (1.0 - theta) : theta), pll, fgp * mgp * id->GENOTYPE[G[0] + G[1]] * rgf[P[0] + P[1]] * (id->PHENOTYPE == 2 ? pen[P[0] + P[1]] : (1.0 - pen[P[0] + P[1]])) * (fv[0] == fv[1] ? (1.0 - theta) : theta) * (mv[0] == mv[1] ? (1.0 - theta) : theta));
//										}
									}
								}
							}
						}
					}
				}
			}
		}

		id->part_likelihood *= pll;printf("%lf\n", pll);

		return;
	}

	id = dip[founder->data[fdigit]].data;
	id->part_likelihood = 1.0;
	for(i = 0; i < id->num_of_childs; i++){
		Cid = id->childs[i];
		id->part_likelihood *= Cid->part_likelihood;
	}

	pll = 0;
	for(H[0] = 0; H[0] < Hlimit[0]; H[0]++){
		G[0] = H[0] <= 1 ? 0 : 1;
		G[1] = H[0] % 2 == 0 ? 0 : 1;
		for(H[1] = 0; H[1] < Hlimit[1]; H[1]++){
			P[0] = H[1] <= 1 ? 0 : 1;
			P[1] = H[1] % 2 == 0 ? 0 : 1;

			pll += id->GENOTYPE[G[0] + G[1]] * (id->PHENOTYPE == 2 ? pen[P[0] + P[1]] : (1.0 - pen[P[0] + P[1]])) * rgf[P[0] + P[1]];
		}
	}

	id->part_likelihood *= pll;

	if(fdigit == 0 && cdigit == 0){
		res->result[0] = 1.0;
		for(i = 0; i < founder->number; i++){
			res->result[0] *= dip[founder->data[i]].data->part_likelihood;
		}
		for(i = 0; i < child->number; i++){
			res->result[0] *= dip[child->data[i]].data->part_likelihood;
		}
	}

	return;
}


void get_likelihood_unite(RDATA *res, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	int dg;
	int tdg;

	int cdigit;
	int cmax_stack = 100;
	LCHDSTACK cstack[cmax_stack];
	int csp = 0;
	int k;

	int fdigit;
	int fmax_stack = 100;
	LFNDSTACK fstack[fmax_stack];
	int fsp = 0;

	fstack[fsp].elv = 1.0;
	fdigit = 0;

FREC:;
	if(fdigit == founder->number){
		csp = 0;
		cstack[csp].elv = fstack[fsp].elv;
		cdigit = 0;

		goto CREC;
		//get_likelihood_child_loop(0, res, fstack[fsp].elv, fd, dip, founder, child, number, pen, theta, raf, rgf);

FRETURN:;

		fsp--;
		fdigit--;
		goto FBACK;

		//return;
	}

	fstack[fsp].id = founder->data[fdigit];

	//ORDERED 0:AA 1:AB 2:BA 3:BB / GENOTYPE 0:AA 1:AB 2:BB / ALLELE 0:A 1:B
	for(fstack[fsp].i = 0; fstack[fsp].i < 4; fstack[fsp].i++){
		dip[fstack[fsp].id].G = fstack[fsp].i == 0 ? 0 : fstack[fsp].i == 3 ? 2 : 1;
		dip[fstack[fsp].id].D[0][0] = fstack[fsp].i == 0 ? 0 : fstack[fsp].i == 1 ? 0 : 1;
		dip[fstack[fsp].id].D[1][0] = fstack[fsp].i == 0 ? 0 : fstack[fsp].i == 2 ? 0 : 1;

		//dip[id].G = i;
		//dip[id].D[0][0] = i == 2 ? 1 : 0;
		//dip[id].D[1][0] = i == 0 ? 0 : 1;

		if(dip[fstack[fsp].id].data->GENOTYPE[dip[fstack[fsp].id].G] == 0){
			continue;
		}

		fstack[fsp].ttelv = fstack[fsp].elv * dip[fstack[fsp].id].data->GENOTYPE[dip[fstack[fsp].id].G];

		//ORDERED 0:NN 1:NS 2:SN 3:SS / GENOTYPE 0:NN 1:NS 2:SS /ALLELE 0:N 1:S
		for(fstack[fsp].j = 0; fstack[fsp].j < 4; fstack[fsp].j++){
			dip[fstack[fsp].id].P = fstack[fsp].j == 0 ? 0 : fstack[fsp].j == 3 ? 2 : 1;
			dip[fstack[fsp].id].D[0][1] = fstack[fsp].j == 0 ? 0 : fstack[fsp].j == 1 ? 0 : 1;
			dip[fstack[fsp].id].D[1][1] = fstack[fsp].j == 0 ? 0 : fstack[fsp].j == 2 ? 0 : 1;

			/*
			telv *= dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]);
			telv *= rgf[dip[id].P]m//dip[id].P == 2 ? raf* raf : dip[id].P == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf);
			*/
			fstack[fsp].telv = fstack[fsp].ttelv * rgf[dip[fstack[fsp].id].P] * (dip[fstack[fsp].id].data->PHENOTYPE == 2 ? pen[dip[fstack[fsp].id].P] : (1 - pen[dip[fstack[fsp].id].P]));
			//printf("GEN FREQ:%d:%lg:%lg\n", dip[id].G, raf,(dip[id].G == 2 ? raf * raf : dip[id].G == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf)));

			if(fstack[fsp].telv == 0){
				continue;
			}


			fsp++;
			fstack[fsp].elv = fstack[fsp - 1].telv;

			fdigit++;
			goto FREC;

			//get_likelihood_fnd_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
FBACK:;
		}
	}

	if(fsp > 0){
		fsp--;
		fdigit--;
		goto FBACK;
	}

	return;

CREC:;
	//printf("CHD_START:%d-%d\n", fdigit, cdigit);
	if(cdigit == child->number){
		dg = (int)log10(cstack[csp].elv);//(log(elv) / log(10));
		dg = dg < 0 ? abs(dg) + 1 : 0;
		if(dg >= res->max){
			if((res->result = (double *)realloc(res->result, sizeof(double) * (dg + 1))) == NULL){
				return;
			}

			for(; res->max < dg + 1; res->max++){
				res->result[res->max] = 0;
			}
		}

		res->result[dg] += cstack[csp].elv;
		tdg = (int)log10(res->result[dg]);//(log(elv) / log(10));
		tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		while(dg != tdg){
			res->result[tdg] += res->result[dg];
			res->result[dg] = 0;

			dg = tdg;
			tdg = (int)log10(res->result[dg]);//(log(dg) / log(10));
			tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		}

		//do{
			csp--;
			cdigit--;
		//} while(cstack[csp].fv[0] + 1 == cstack[csp].flimit[0] && cstack[csp].fv[1] + 1 == cstack[csp].flimit[1] && cstack[csp].mv[0] + 1 == cstack[csp].mlimit[0] && cstack[csp].mv[1] + 1 == cstack[csp].mlimit[1]);

		goto CBACK;
		//return;
	}

	for(k = 0; k < 2; k++){
		cstack[csp].flimit[k] = 2;
		cstack[csp].mlimit[k] = 2;
		cstack[csp].frcomb[k] = k == 0 ? theta : (1 - theta);
		cstack[csp].mrcomb[k] = k == 0 ? theta : (1 - theta);
		cstack[csp].dammy_g[k] = k;
		cstack[csp].dammy_p[k] = k;
	}
	//cstack[csp].elv = elv;

	cstack[csp].id = child->data[cdigit];
	cstack[csp].Fid = dip[cstack[csp].id].data->Fid == NULL ? -1 : dip[cstack[csp].id].data->Fid->dip;
	cstack[csp].Mid = dip[cstack[csp].id].data->Mid == NULL ? -1 : dip[cstack[csp].id].data->Mid->dip;

	if(cstack[csp].Fid != -1){
		if(dip[cstack[csp].Fid].D[0][0] == dip[cstack[csp].Fid].D[1][0]){
			cstack[csp].flimit[0] = 1;
			cstack[csp].rc = cstack[csp].frcomb[0] + cstack[csp].frcomb[1];
			cstack[csp].frcomb[0] = cstack[csp].rc;
			cstack[csp].frcomb[1] = cstack[csp].rc;
		}
		if(dip[cstack[csp].Fid].D[0][1] == dip[cstack[csp].Fid].D[1][1]){
			cstack[csp].flimit[1] = 1;
			cstack[csp].rc = cstack[csp].frcomb[0] + cstack[csp].frcomb[1];
			cstack[csp].frcomb[0] = cstack[csp].rc;
			cstack[csp].frcomb[1] = cstack[csp].rc;
		}
	}
	if(cstack[csp].Mid != -1){
		if(dip[cstack[csp].Mid].D[0][0] == dip[cstack[csp].Mid].D[1][0]){
			cstack[csp].mlimit[0] = 1;
			cstack[csp].rc = cstack[csp].mrcomb[0] + cstack[csp].mrcomb[1];
			cstack[csp].mrcomb[0] = cstack[csp].rc;
			cstack[csp].mrcomb[1] = cstack[csp].rc;
		}
		if(dip[cstack[csp].Mid].D[0][1] == dip[cstack[csp].Mid].D[1][1]){
			cstack[csp].mlimit[1] = 1;
			cstack[csp].rc = cstack[csp].mrcomb[0] + cstack[csp].mrcomb[1];
			cstack[csp].mrcomb[0] = cstack[csp].rc;
			cstack[csp].mrcomb[1] = cstack[csp].rc;
		}
	}

	for(cstack[csp].fv[0] = 0; cstack[csp].fv[0] < cstack[csp].flimit[0]; cstack[csp].fv[0]++){
		dip[cstack[csp].id].D[0][0] = cstack[csp].Fid != -1 ? dip[cstack[csp].Fid].D[cstack[csp].fv[0]][0] : cstack[csp].dammy_g[cstack[csp].fv[0]];
		for(cstack[csp].mv[0] = 0; cstack[csp].mv[0] < cstack[csp].mlimit[0]; cstack[csp].mv[0]++){
			dip[cstack[csp].id].D[1][0] = cstack[csp].Mid != -1 ? dip[cstack[csp].Mid].D[cstack[csp].mv[0]][0] : cstack[csp].dammy_g[cstack[csp].mv[0]];

			dip[cstack[csp].id].G = dip[cstack[csp].id].D[0][0] + dip[cstack[csp].id].D[1][0];
			cstack[csp].ttelv[0] = cstack[csp].elv * dip[cstack[csp].id].data->GENOTYPE[dip[cstack[csp].id].G];

			if(cstack[csp].ttelv[0] == 0){
				continue;
			}

			for(cstack[csp].fv[1] = 0; cstack[csp].fv[1] < cstack[csp].flimit[1]; cstack[csp].fv[1]++){
				dip[cstack[csp].id].D[0][1] = cstack[csp].Fid != -1 ? dip[cstack[csp].Fid].D[cstack[csp].fv[1]][1] : cstack[csp].dammy_p[cstack[csp].fv[1]];

				cstack[csp].ttelv[1] = cstack[csp].ttelv[0] * (cstack[csp].Fid != -1 ? cstack[csp].frcomb[cstack[csp].fv[0] == cstack[csp].fv[1] ? 1 : 0] : raf[dip[cstack[csp].id].D[0][1]]);//1.0;

				if(cstack[csp].ttelv[1] == 0){
					continue;
				}

				for(cstack[csp].mv[1] = 0; cstack[csp].mv[1] < cstack[csp].mlimit[1]; cstack[csp].mv[1]++){
					dip[cstack[csp].id].D[1][1] = cstack[csp].Mid != -1 ? dip[cstack[csp].Mid].D[cstack[csp].mv[1]][1] : cstack[csp].dammy_p[cstack[csp].mv[1]];

					dip[cstack[csp].id].P = dip[cstack[csp].id].D[0][1] + dip[cstack[csp].id].D[1][1];

					cstack[csp].telv = cstack[csp].ttelv[1] * (dip[cstack[csp].id].data->PHENOTYPE == 2 ? pen[dip[cstack[csp].id].P] : (1 - pen[dip[cstack[csp].id].P])) * (cstack[csp].Mid != -1 ? cstack[csp].mrcomb[cstack[csp].mv[0] == cstack[csp].mv[1] ? 1 : 0] : raf[dip[cstack[csp].id].D[1][1]]);

					if(cstack[csp].telv == 0){
						continue;
					}

					csp++;
					for(k = 0; k < 2; k++){
						cstack[csp].flimit[k] = 2;
						cstack[csp].mlimit[k] = 2;
						cstack[csp].frcomb[k] = k == 0 ? theta : (1 - theta);
						cstack[csp].mrcomb[k] = k == 0 ? theta : (1 - theta);
						cstack[csp].dammy_g[k] = k;
						cstack[csp].dammy_p[k] = k;
					}

					cdigit++;
					cstack[csp].elv = cstack[csp - 1].telv;
					goto CREC;
					//get_likelihood_child_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
CBACK:;
				}
			}
		}
	}


	if(csp > 0){
		//do{
			csp--;
			cdigit--;
		//} while(csp > 0 && cstack[csp].fv[0] + 1 == cstack[csp].flimit[0] && cstack[csp].fv[1] + 1 == cstack[csp].flimit[1] && cstack[csp].mv[0] + 1 == cstack[csp].mlimit[0] && cstack[csp].mv[1] + 1 == cstack[csp].mlimit[1]);
		goto CBACK;
	}
	//return;
	
	goto FRETURN;
}


void get_likelihood_child_loop(int digit, RDATA *res, double elv, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	int dg;
	int tdg;

	int max_stack = 100;
	LCHDSTACK stack[max_stack];
	int sp = 0;
	int i;


	for(i = 0; i < 2; i++){
		stack[sp].flimit[i] = 2;
		stack[sp].mlimit[i] = 2;
		stack[sp].frcomb[i] = i == 0 ? theta : (1 - theta);
		stack[sp].mrcomb[i] = i == 0 ? theta : (1 - theta);
		stack[sp].dammy_g[i] = i;
		stack[sp].dammy_p[i] = i;
	}
	stack[sp].elv = elv;


REC:;
	if(digit == child->number){
		dg = (int)log10(stack[sp].elv);//(log(elv) / log(10));
		dg = dg < 0 ? abs(dg) + 1 : 0;
		if(dg >= res->max){
			if((res->result = (double *)realloc(res->result, sizeof(double) * (dg + 1))) == NULL){
				return;
			}

			for(; res->max < dg + 1; res->max++){
				res->result[res->max] = 0;
			}
		}

		res->result[dg] += stack[sp].elv;
		tdg = (int)log10(stack[sp].elv);//(log(elv) / log(10));
		tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		while(dg != tdg){
			res->result[tdg] += res->result[dg];
			res->result[dg] = 0;

			dg = tdg;
			tdg = (int)log(dg);//(log(dg) / log(10));
			tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		}

		do{
			sp--;
			digit--;
		} while(stack[sp].fv[0] + 1 == stack[sp].flimit[0] && stack[sp].fv[1] + 1 == stack[sp].flimit[1] && stack[sp].mv[0] + 1 == stack[sp].mlimit[0] && stack[sp].mv[1] + 1 == stack[sp].mlimit[1]);

		goto BACK;
		//return;
	}

	stack[sp].id = child->data[digit];
	stack[sp].Fid = dip[stack[sp].id].data->Fid == NULL ? -1 : dip[stack[sp].id].data->Fid->dip;
	stack[sp].Mid = dip[stack[sp].id].data->Mid == NULL ? -1 : dip[stack[sp].id].data->Mid->dip;

	if(stack[sp].Fid != -1){
		if(dip[stack[sp].Fid].D[0][0] == dip[stack[sp].Fid].D[1][0]){
			stack[sp].flimit[0] = 1;
			stack[sp].rc = stack[sp].frcomb[0] + stack[sp].frcomb[1];
			stack[sp].frcomb[0] = stack[sp].rc;
			stack[sp].frcomb[1] = stack[sp].rc;
		}
		if(dip[stack[sp].Fid].D[0][1] == dip[stack[sp].Fid].D[1][1]){
			stack[sp].flimit[1] = 1;
			stack[sp].rc = stack[sp].frcomb[0] + stack[sp].frcomb[1];
			stack[sp].frcomb[0] = stack[sp].rc;
			stack[sp].frcomb[1] = stack[sp].rc;
		}
	}
	if(stack[sp].Mid != -1){
		if(dip[stack[sp].Mid].D[0][0] == dip[stack[sp].Mid].D[1][0]){
			stack[sp].mlimit[0] = 1;
			stack[sp].rc = stack[sp].mrcomb[0] + stack[sp].mrcomb[1];
			stack[sp].mrcomb[0] = stack[sp].rc;
			stack[sp].mrcomb[1] = stack[sp].rc;
		}
		if(dip[stack[sp].Mid].D[0][1] == dip[stack[sp].Mid].D[1][1]){
			stack[sp].mlimit[1] = 1;
			stack[sp].rc = stack[sp].mrcomb[0] + stack[sp].mrcomb[1];
			stack[sp].mrcomb[0] = stack[sp].rc;
			stack[sp].mrcomb[1] = stack[sp].rc;
		}
	}

	for(stack[sp].fv[0] = 0; stack[sp].fv[0] < stack[sp].flimit[0]; stack[sp].fv[0]++){
		dip[stack[sp].id].D[0][0] = stack[sp].Fid != -1 ? dip[stack[sp].Fid].D[stack[sp].fv[0]][0] : stack[sp].dammy_g[stack[sp].fv[0]];
		for(stack[sp].mv[0] = 0; stack[sp].mv[0] < stack[sp].mlimit[0]; stack[sp].mv[0]++){
			dip[stack[sp].id].D[1][0] = stack[sp].Mid != -1 ? dip[stack[sp].Mid].D[stack[sp].mv[0]][0] : stack[sp].dammy_g[stack[sp].mv[0]];

			dip[stack[sp].id].G = dip[stack[sp].id].D[0][0] + dip[stack[sp].id].D[1][0];
			stack[sp].ttelv[0] = stack[sp].elv * dip[stack[sp].id].data->GENOTYPE[dip[stack[sp].id].G];

			if(stack[sp].ttelv[0] == 0){
				continue;
			}

			for(stack[sp].fv[1] = 0; stack[sp].fv[1] < stack[sp].flimit[1]; stack[sp].fv[1]++){
				dip[stack[sp].id].D[0][1] = stack[sp].Fid != -1 ? dip[stack[sp].Fid].D[stack[sp].fv[1]][1] : stack[sp].dammy_p[stack[sp].fv[1]];

				stack[sp].ttelv[1] = stack[sp].ttelv[0] * (stack[sp].Fid != -1 ? stack[sp].frcomb[stack[sp].fv[0] == stack[sp].fv[1] ? 1 : 0] : raf[dip[stack[sp].id].D[0][1]]);//1.0;

				if(stack[sp].ttelv[1] == 0){
					continue;
				}

				for(stack[sp].mv[1] = 0; stack[sp].mv[1] < stack[sp].mlimit[1]; stack[sp].mv[1]++){
					dip[stack[sp].id].D[1][1] = stack[sp].Mid != -1 ? dip[stack[sp].Mid].D[stack[sp].mv[1]][1] : stack[sp].dammy_p[stack[sp].mv[1]];

					dip[stack[sp].id].P = dip[stack[sp].id].D[0][1] + dip[stack[sp].id].D[1][1];

					stack[sp].telv = stack[sp].ttelv[1] * (dip[stack[sp].id].data->PHENOTYPE == 2 ? pen[dip[stack[sp].id].P] : (1 - pen[dip[stack[sp].id].P])) * (stack[sp].Mid != -1 ? stack[sp].mrcomb[stack[sp].mv[0] == stack[sp].mv[1] ? 1 : 0] : raf[dip[stack[sp].id].D[1][1]]);

					if(stack[sp].telv == 0){
						continue;
					}

					sp++;
					for(i = 0; i < 2; i++){
						stack[sp].flimit[i] = 2;
						stack[sp].mlimit[i] = 2;
						stack[sp].frcomb[i] = i == 0 ? theta : (1 - theta);
						stack[sp].mrcomb[i] = i == 0 ? theta : (1 - theta);
						stack[sp].dammy_g[i] = i;
						stack[sp].dammy_p[i] = i;
					}

					digit++;
					stack[sp].elv = stack[sp - 1].telv;
					goto REC;
					//get_likelihood_child_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
BACK:;
				}
			}
		}
	}


	if(sp > 0){
		do{
			sp--;
			digit--;
		} while(sp > 0 && stack[sp].fv[0] + 1 == stack[sp].flimit[0] && stack[sp].fv[1] + 1 == stack[sp].flimit[1] && stack[sp].mv[0] + 1 == stack[sp].mlimit[0] && stack[sp].mv[1] + 1 == stack[sp].mlimit[1]);
		goto BACK;
	}
	return;
}


void get_likelihood_fnd_loop(int digit, RDATA *res, double elv, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	/*
	int id;
	double ttelv;
	double telv;
	int i, j;
	*/

	int max_stack = 100;
	LFNDSTACK stack[max_stack];
	int sp = 0;

	stack[sp].elv = elv;

REC:;

	if(digit == founder->number){
		get_likelihood_child_loop(0, res, stack[sp].elv, fd, dip, founder, child, number, pen, theta, raf, rgf);

		sp--;
		digit--;
		goto BACK;

		//return;
	}

	stack[sp].id = founder->data[digit];

	//ORDERED 0:AA 1:AB 2:BA 3:BB / GENOTYPE 0:AA 1:AB 2:BB / ALLELE 0:A 1:B
	for(stack[sp].i = 0; stack[sp].i < 4; stack[sp].i++){
		dip[stack[sp].id].G = stack[sp].i == 0 ? 0 : stack[sp].i == 3 ? 2 : 1;
		dip[stack[sp].id].D[0][0] = stack[sp].i == 0 ? 0 : stack[sp].i == 1 ? 0 : 1;
		dip[stack[sp].id].D[1][0] = stack[sp].i == 0 ? 0 : stack[sp].i == 2 ? 0 : 1;

		//dip[id].G = i;
		//dip[id].D[0][0] = i == 2 ? 1 : 0;
		//dip[id].D[1][0] = i == 0 ? 0 : 1;

		if(dip[stack[sp].id].data->GENOTYPE[dip[stack[sp].id].G] == 0){
			continue;
		}

		stack[sp].ttelv = stack[sp].elv * dip[stack[sp].id].data->GENOTYPE[dip[stack[sp].id].G];

		//ORDERED 0:NN 1:NS 2:SN 3:SS / GENOTYPE 0:NN 1:NS 2:SS /ALLELE 0:N 1:S
		for(stack[sp].j = 0; stack[sp].j < 4; stack[sp].j++){
			dip[stack[sp].id].P = stack[sp].j == 0 ? 0 : stack[sp].j == 3 ? 2 : 1;
			dip[stack[sp].id].D[0][1] = stack[sp].j == 0 ? 0 : stack[sp].j == 1 ? 0 : 1;
			dip[stack[sp].id].D[1][1] = stack[sp].j == 0 ? 0 : stack[sp].j == 2 ? 0 : 1;

			/*
			telv *= dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]);
			telv *= rgf[dip[id].P];//dip[id].P == 2 ? raf* raf : dip[id].P == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf);
			*/
			stack[sp].telv = stack[sp].ttelv * rgf[dip[stack[sp].id].P] * (dip[stack[sp].id].data->PHENOTYPE == 2 ? pen[dip[stack[sp].id].P] : (1 - pen[dip[stack[sp].id].P]));
			//printf("GEN FREQ:%d:%lg:%lg\n", dip[id].G, raf,(dip[id].G == 2 ? raf * raf : dip[id].G == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf)));

			if(stack[sp].telv == 0){
				continue;
			}


			sp++;
			stack[sp].elv = stack[sp - 1].telv;

			digit++;
			goto REC;

			//get_likelihood_fnd_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
BACK:;
		}
	}

	if(sp > 0){
		sp--;
		digit--;
		goto BACK;
	}

	return;
}


void get_likelihood_child_rec(int digit, RDATA *res, double elv, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	int id;
	int Fid;
	int Mid;

	int fv[2];
	int mv[2];
	int flimit[2] = {2, 2};
	int mlimit[2] = {2, 2};
	double frcomb[2] = {theta, (1 - theta)};
	double mrcomb[2] = {theta, (1 - theta)};
	int dammy_g[2] = {0, 1};
	int dammy_p[2] = {0, 1};
	double rc;

	int dg;
	int tdg;

	double telv;
	//double ttelv[2];

	double multi_rcomb[2][2];
	int i, j;


	if(digit == child->number){
		dg = (int)log10(elv);//(log(elv) / log(10));
		dg = dg < 0 ? abs(dg) + 1 : 0;
		if(dg >= res->max){
			if((res->result = (double *)realloc(res->result, sizeof(double) * (dg + 1))) == NULL){
				return;
			}

			for(; res->max < dg + 1; res->max++){
				res->result[res->max] = 0;
			}
		}

		res->result[dg] += elv;
		tdg = (int)log10(elv);//(log(elv) / log(10));
		tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		while(dg != tdg){
			res->result[tdg] += res->result[dg];
			res->result[dg] = 0;

			dg = tdg;
			tdg = (int)log(dg);//(log(dg) / log(10));
			tdg = tdg < 0 ? abs(tdg) + 1 : 0;
		}
		//printf("ADD RESULT(CHD):%d <> %d:%lg:%lg:%lg:%lg:%lg:%lg:%d\n", digit, child->number - 1, telv, res->result[dg], dip[id].data->GENOTYPE[dip[id].G], dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]), frcomb[fv[0] == fv[1] ? 1 : 0], mrcomb[mv[0] == mv[1] ? 1 : 0], dg);

		return;
	}

	id = child->data[digit];
	Fid = dip[id].data->Fid == NULL ? -1 : dip[id].data->Fid->dip;
	Mid = dip[id].data->Mid == NULL ? -1 : dip[id].data->Mid->dip;

	if(Fid != -1){
		if(dip[Fid].D[0][0] == dip[Fid].D[1][0]){
			flimit[0] = 1;
			rc = frcomb[0] + frcomb[1];
			frcomb[0] = rc;
			frcomb[1] = rc;
		}
		if(dip[Fid].D[0][1] == dip[Fid].D[1][1]){
			flimit[1] = 1;
			rc = frcomb[0] + frcomb[1];
			frcomb[0] = rc;
			frcomb[1] = rc;
		}
	}
	if(Mid != -1){
		if(dip[Mid].D[0][0] == dip[Mid].D[1][0]){
			mlimit[0] = 1;
			rc = mrcomb[0] + mrcomb[1];
			mrcomb[0] = rc;
			mrcomb[1] = rc;
		}
		if(dip[Mid].D[0][1] == dip[Mid].D[1][1]){
			mlimit[1] = 1;
			rc = mrcomb[0] + mrcomb[1];
			mrcomb[0] = rc;
			mrcomb[1] = rc;
		}
	}

	for(i = 0; i < 2; i++){
		for(j = 0; j < 2; j++){
			multi_rcomb[i][j] = frcomb[i] * mrcomb[j];
		}
	}

	for(fv[0] = 0; fv[0] < flimit[0]; fv[0]++){
		dip[id].D[0][0] = Fid != -1 ? dip[Fid].D[fv[0]][0] : dammy_g[fv[0]];
		for(mv[0] = 0; mv[0] < mlimit[0]; mv[0]++){
			dip[id].D[1][0] = Mid != -1 ? dip[Mid].D[mv[0]][0] : dammy_g[mv[0]];

			dip[id].G = dip[id].D[0][0] + dip[id].D[1][0];
			//ttelv[0] = elv * dip[id].data->GENOTYPE[dip[id].G];
/*
			if(ttelv[0] == 0){
				continue;
			}
*/
			if(dip[id].data->GENOTYPE[dip[id].G] == 0){
				continue;
			}

			for(fv[1] = 0; fv[1] < flimit[1]; fv[1]++){
				dip[id].D[0][1] = Fid != -1 ? dip[Fid].D[fv[1]][1] : dammy_p[fv[1]];

				//ttelv[1] = ttelv[0] * (Fid != -1 ? frcomb[fv[0] == fv[1] ? 1 : 0] : raf[dip[id].D[0][1]]);//1.0;

				if((Fid != -1 ? frcomb[fv[0] == fv[1] ? 1 : 0] : raf[dip[id].D[0][1]]) == 0){
					continue;
				}
				//ttelv[0] = elv * (Fid != -1 ? frcomb[fv[0] == fv[1] ? 1 : 0] : raf[dip[id].D[0][1]]);//1.0;
/*
				if(ttelv[1] == 0){
					continue;
				}
*/

				for(mv[1] = 0; mv[1] < mlimit[1]; mv[1]++){
					dip[id].D[1][1] = Mid != -1 ? dip[Mid].D[mv[1]][1] : dammy_p[mv[1]];

					dip[id].P = dip[id].D[0][1] + dip[id].D[1][1];

					/*
					telv *= dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]);
					telv *= Mid != -1 ? mrcomb[mv[0] == mv[1] ? 1 : 0] : raf[dip[id].D[1][1]];//1.0;
					*/
					//telv = ttelv[1] * (dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P])) * (Mid != -1 ? mrcomb[mv[0] == mv[1] ? 1 : 0] : raf[dip[id].D[1][1]]);
					if((Mid != -1 ? mrcomb[mv[0] == mv[1] ? 1 : 0] : raf[dip[id].D[1][1]]) == 0 || dip[id].data->calculate[dip[id].G][dip[id].P] == 0){
						continue;
					}
					telv = elv * dip[id].data->calculate[dip[id].G][dip[id].P] * (Fid == -1 ? Mid == -1 ?  raf[dip[id].D[0][1]] * raf[dip[id].D[1][1]] : raf[dip[id].D[0][1]] : Mid == -1 ? raf[dip[id].D[1][1]] : multi_rcomb[fv[0] == fv[1] ? 1 : 0][mv[0] == mv[1] ? 1 : 0]);

/*
					if(telv == 0){
						//printf("INTERAPT:%d <> %d:%lg:%lg:%lg:%lg\n", digit, child->number, dip[id].data->GENOTYPE[dip[id].G], dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]), frcomb[fv[0] == fv[1] ? 1 : 0], mrcomb[mv[0] == mv[1] ? 1 : 0]);
						continue;
					}
*/

					get_likelihood_child_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
				}
			}
		}
	}

	return;
}


void get_likelihood_fnd_rec(int digit, RDATA *res, double elv, FAMILY_DATA *fd, DIP *dip, ADATA *founder, ADATA *child, int number, double *pen, double theta, double *raf, double *rgf){
	int id;
	//double ttelv;
	double telv;
	int i, j;


	if(digit == founder->number){
		get_likelihood_child_rec(0, res, elv, fd, dip, founder, child, number, pen, theta, raf, rgf);

		return;
	}

	id = founder->data[digit];

	//ORDERED 0:AA 1:AB 2:BA 3:BB / GENOTYPE 0:AA 1:AB 2:BB / ALLELE 0:A 1:B
	for(i = 0; i < 4; i++){
		dip[id].G = i == 0 ? 0 : i == 3 ? 2 : 1;
		dip[id].D[0][0] = i == 0 ? 0 : i == 1 ? 0 : 1;
		dip[id].D[1][0] = i == 0 ? 0 : i == 2 ? 0 : 1;

		//dip[id].G = i;
		//dip[id].D[0][0] = i == 2 ? 1 : 0;
		//dip[id].D[1][0] = i == 0 ? 0 : 1;

		if(dip[id].data->GENOTYPE[dip[id].G] == 0){
			continue;
		}

		//ttelv = elv * dip[id].data->GENOTYPE[dip[id].G];

		//ORDERED 0:NN 1:NS 2:SN 3:SS / GENOTYPE 0:NN 1:NS 2:SS /ALLELE 0:N 1:S
		for(j = 0; j < 4; j++){
			dip[id].P = j == 0 ? 0 : j == 3 ? 2 : 1;
			dip[id].D[0][1] = j == 0 ? 0 : j == 1 ? 0 : 1;
			dip[id].D[1][1] = j == 0 ? 0 : j == 2 ? 0 : 1;

			/*
			telv *= dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]);
			telv *= rgf[dip[id].P];//dip[id].P == 2 ? raf* raf : dip[id].P == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf);
			*/
			//telv = ttelv * rgf[dip[id].P] * (dip[id].data->PHENOTYPE == 2 ? pen[dip[id].P] : (1 - pen[dip[id].P]));
			telv = elv * rgf[dip[id].P] * dip[id].data->calculate[dip[id].G][dip[id].P];
			//printf("GEN FREQ:%d:%lg:%lg\n", dip[id].G, raf,(dip[id].G == 2 ? raf * raf : dip[id].G == 1 ? raf * (1.0 - raf) : (1.0 - raf) * (1.0 - raf)));

			if(telv == 0){
				continue;
			}

			get_likelihood_fnd_rec(digit + 1, res, telv, fd, dip, founder, child, number, pen, theta, raf, rgf);
		}
	}

	return;
}


ADATA *get_founder(DIP *dip, int number){
	ADATA *fnd;

	int i;


	if((fnd = (ADATA *)malloc(sizeof(ADATA))) == NULL){
		return(NULL);
	}

	if((fnd->data = (int *)malloc(sizeof(int) * number)) == NULL){
		return(NULL);
	}
	fnd->number = 0;

	for(i = 0; i < number; i++){
		if(dip[i].data->Fid == NULL && dip[i].data->Mid == NULL){
			fnd->data[fnd->number++] = i;
		}
	}

	return fnd;
}


ADATA *get_child(DIP *dip, int number, ADATA *fnd){
	ADATA *chd;
	ADATA *rest;


	int i, j, k, m, n;


	if((chd = (ADATA *)malloc(sizeof(ADATA))) == NULL){
		return(NULL);
	}
	if((chd->data = (int *)malloc(sizeof(int) * number)) == NULL){
		return(NULL);
	}
	chd->number = 0;

	if((rest = (ADATA *)malloc(sizeof(ADATA))) == NULL){
		return(NULL);
	}
	if((rest->data = (int *)malloc(sizeof(int) * number)) == NULL){
		return(NULL);
	}
	rest->number = 0;

	for(i = 0; i < number; i++){
		if(dip[i].data->Fid != NULL || dip[i].data->Mid != NULL){
			rest->data[rest->number++] = i;
		}
	}

	while(chd->number < rest->number){
		//printf("CHD:REST:%d <> %d\n", chd->number, rest->number);
		for(i = 0; i < rest->number; i++){
			if(rest->data[i] == -1){
				continue;
			}

			for(j = 0; j < fnd->number; j++){
				if(dip[rest->data[i]].data->Fid == NULL || dip[rest->data[i]].data->Fid->dip == fnd->data[j]){
					break;
				}
			}
			for(k = 0; k < fnd->number; k++){
				if(dip[rest->data[i]].data->Mid == NULL || dip[rest->data[i]].data->Mid->dip == fnd->data[k]){
					break;
				}
			}
			for(m = 0; m < chd->number; m++){
				if(dip[rest->data[i]].data->Fid == NULL || dip[rest->data[i]].data->Fid->dip == chd->data[m]){
					break;
				}
			}
			for(n = 0; n < chd->number; n++){
				if(dip[rest->data[i]].data->Mid == NULL || dip[rest->data[i]].data->Mid->dip == chd->data[n]){
					break;
				}
			}
			if((j < fnd->number || m < chd->number) && (k < fnd->number || n < chd->number)){
				chd->data[chd->number++] = rest->data[i];
				rest->data[i] = -1;
			}
		}
	}

	free(rest->data);
	free(rest);

	return(chd);
}


int cnt_pedigree(FAMILY_DATA *fd){
	int cnt;
	FAMILY_DATA *cur;

	cnt = 0;
	cur = fd;
	while(cur != NULL){
		cnt++;
		cur = cur->next;
	}

	return cnt;
}


FAMILY_DATA *get_genotype_data(char *in_file_name, FAMILY_DATA *fd){
	FILE *fp;

	char str[2048];
	char *word;
	char *new_word;

	int max_element;
	int num_of_element;
	char **element;

	FAMILY_DATA *cur;
	char *id;

	int i;


	if((fp = fopen(in_file_name, "r")) == NULL){
		return(NULL);
	}

	fgets(str, sizeof(str), fp);//HEADER
	max_element = 10;
	if((element = (char **)malloc(sizeof(char *) * max_element)) == NULL){
		printf("Flag1\n");
		return(NULL);
	}

	while((fgets(str, sizeof(str), fp)) != NULL){
		num_of_element = 0;
		word = str;
		new_word = strstr(word, "\n");
		new_word[0] = '\0';
		while((new_word = strstr(word, ",")) != NULL){
			new_word = strstr(word, ",");
			new_word[0] = '\0';
			new_word++;

			if(num_of_element >= max_element){
				max_element += max_element;
				if((element = (char **)realloc(element, sizeof(char *) * max_element)) == NULL){
					printf("Flag2\n");
					return(NULL);
				}
			}

			element[num_of_element++] = word;

			word = new_word;
		}
		element[num_of_element++] = word;

		id = element[0];
		cur = fd->next;
		while(cur != NULL){
			if((strcmp(id, cur->id)) == 0){
				break;
			}

			cur = cur->next;
		}
		if(cur != NULL){
			for(i = 0; i < 3; i++){
				sscanf(element[i + 1], "%lg", &(cur->GENOTYPE[i]));
			}
		}
	}

	free(element);
	fclose(fp);

	return fd;
}


FAMILY_DATA *get_pedigree_data(char *in_file_name){
	FILE *fp;

	char str[2048];
	char *word;
	char *new_word;

	int max_element;
	int num_of_element;
	char **element;

	FAMILY_DATA *init;
	FAMILY_DATA *cur;
	FAMILY_DATA *check;

	//int i;


	if((init = (FAMILY_DATA *)malloc(sizeof(FAMILY_DATA))) == NULL){
		printf("Flag1\n");
		return(NULL);
	}
	init->next = NULL;

	if((fp = fopen(in_file_name, "r")) == NULL){
		printf("Flag2:%s\n", in_file_name);
		return(NULL);
	}

	//fgets(str, sizeof(str), fp);//HEADER
	max_element = 10;
	if((element = (char **)malloc(sizeof(char *) * max_element)) == NULL){
		printf("Flag3\n");
		return(NULL);
	}
	cur = init;
	while((fgets(str, sizeof(str), fp)) != NULL){
		num_of_element = 0;
		word = str;
		new_word = strstr(word, "\n");
		new_word[0] = '\0';
		while((new_word = strstr(word, ",")) != NULL){
			new_word = strstr(word, ",");
			new_word[0] = '\0';
			new_word++;

			if(num_of_element >= max_element){
				max_element += max_element;
				if((element = (char **)realloc(element, sizeof(char *) * max_element)) == NULL){
					printf("Flag4\n");
					return(NULL);
				}
			}

			element[num_of_element++] = word;

			word = new_word;
		}
		element[num_of_element++] = word;

		//LOC and ID
		check = init;
		cur = init->next;
		while(cur != NULL){
			if(strcmp(cur->id, element[1]) == 0){
				break;
			}

			check = cur;
			cur = cur->next;
		}
		if(cur == NULL){
			cur = check;

			if((cur->next = mk_new_ind(element[1])) == NULL){
				printf("Flag5\n");
				return(NULL);
			}
			cur = cur->next;
		}

		//SEX
		sscanf(element[4], "%d", &(cur->SEX));

		//PHENOTYPE
		sscanf(element[5], "%d", &(cur->PHENOTYPE));

		//Fid
		if(strcmp(element[2], "0") == 0){
			cur->Fid = NULL;
		}
		else{
			check = init->next;
			while(check != NULL){
				if(strcmp(element[2], check->id) == 0){
					break;
				}

				check = check->next;
			}
			if(check == NULL){
				check = init;
				while(check->next != NULL){
					check = check->next;
				}
				if((check->next = mk_new_ind(element[2])) == NULL){
					printf("Flag6\n");
					return(NULL);
				}

				check = check->next;
			}

			cur->Fid = check;
		}

		//Mid
		if(strcmp(element[3], "0") == 0){
			cur->Mid = NULL;
		}
		else{
			check = init->next;
			while(check != NULL){
				if(strcmp(element[3], check->id) == 0){
					break;
				}

				check = check->next;
			}
			if(check == NULL){
				check = init;
				while(check->next != NULL){
					check = check->next;
				}
				if((check->next = mk_new_ind(element[3])) == NULL){
					printf("Flag7\n");
					return(NULL);
				}

				check = check->next;
			}

			cur->Mid = check;
		}

	}

	cur = init->next;
	while(cur != NULL){
		if(cur->Fid != NULL){
			if(cur->Fid->num_of_childs >= cur->Fid->max_childs){
				cur->Fid->max_childs += cur->Fid->max_childs;
				if((cur->Fid->childs = (FAMILY_DATA **)realloc(cur->Fid->childs, sizeof(FAMILY_DATA *) * cur->Fid->max_childs)) == NULL){
					printf("Flag8\n");
					return(NULL);
				}
			}

			cur->Fid->childs[cur->Fid->num_of_childs++] = cur;
		}
		if(cur->Mid != NULL){
			if(cur->Mid->num_of_childs >= cur->Mid->max_childs){
				cur->Mid->max_childs += cur->Mid->max_childs;
				if((cur->Mid->childs = (FAMILY_DATA **)realloc(cur->Mid->childs, sizeof(FAMILY_DATA *) * cur->Mid->max_childs)) == NULL){
					printf("Flag9\n");
					return(NULL);
				}
			}

			cur->Mid->childs[cur->Mid->num_of_childs++] = cur;
		}

		cur = cur->next;
	}

	free(element);
	fclose(fp);

	return(init);
}


FAMILY_DATA *mk_new_ind(char *id){
	FAMILY_DATA *cur;

	int i;

	if((cur = (FAMILY_DATA *)malloc(sizeof(FAMILY_DATA))) == NULL){
		printf("Flag1\n");
		return(NULL);
	}
	cur->next = NULL;

	if((cur->id = (char *)malloc(sizeof(char) * (strlen(id) + 1))) == NULL){
		printf("Flag2\n");
		return(NULL);
	}
	strcpy(cur->id, id);

	for(i = 0; i < 3; i++){
		cur->GENOTYPE[i] = 0;
	}

	cur->childs = NULL;
	cur->max_childs = 5;
	if((cur->childs = (FAMILY_DATA **)malloc(sizeof(FAMILY_DATA *) * cur->max_childs)) == NULL){
		printf("Flag3\n");
		return(NULL);
	}
	for(i = 0; i < cur->max_childs; i++){
		cur->childs[i] = NULL;
	}
	cur->num_of_childs = 0;

	return(cur);
}


double *get_penetrance_data(char *in_file_name){
	FILE *fp;

	char str[2048];
	char *word;
	char *new_word;

	int max_element;
	int num_of_element;
	char **element;

	double *result;

	int i;


	if((result = (double *)malloc(sizeof(double) * 3)) == NULL){
		printf("Flag0\n");
		return(NULL);
	}

	if((fp = fopen(in_file_name, "r")) == NULL){
		return(NULL);
	}

	fgets(str, sizeof(str), fp);//HEADER
	max_element = 10;
	if((element = (char **)malloc(sizeof(char *) * max_element)) == NULL){
		printf("Flag1\n");
		return(NULL);
	}

	while((fgets(str, sizeof(str), fp)) != NULL){
		num_of_element = 0;
		word = str;
		new_word = strstr(word, "\n");
		new_word[0] = '\0';
		while((new_word = strstr(word, ",")) != NULL){
			new_word = strstr(word, ",");
			new_word[0] = '\0';
			new_word++;

			if(num_of_element >= max_element){
				max_element += max_element;
				if((element = (char **)realloc(element, sizeof(char *) * max_element)) == NULL){
					printf("Flag2\n");
					return(NULL);
				}
			}

			element[num_of_element++] = word;

			word = new_word;
		}
		element[num_of_element++] = word;

		for(i = 0; i < 3; i++){
			sscanf(element[2 - i], "%lf", &(result[i]));
		}
	}

	free(element);
	fclose(fp);

	return(result);
}


double *get_risk_allele_frequency(double risk_allele_freq){
	double *result;


	if((result = (double *)malloc(sizeof(double) * 2)) == NULL){
		return(NULL);
	}

	result[0] = 1.0 - risk_allele_freq;
	result[1] = risk_allele_freq;

	return(result);
}


double *get_risk_genotype_frequency(double risk_allele_freq){
	double *result;


	if((result = (double *)malloc(sizeof(double) * 3)) == NULL){
		return(NULL);
	}

	result[0] = (1.0 - risk_allele_freq) * (1.0 - risk_allele_freq);
	result[1] = risk_allele_freq * (1.0 - risk_allele_freq);
	result[2] = risk_allele_freq * risk_allele_freq;

	return(result);
}




